/*******************************************************************************
 * Copyright (c) 2007, 2015 Borland Software Corporation and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Borland Software Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.m2m.internal.qvt.oml.common.io.eclipse;

import org.eclipse.core.runtime.IPath;
import org.eclipse.m2m.internal.qvt.oml.common.io.CResource;


public abstract class AbstractBundleResource implements CResource {
	
	protected final IPath resourcePath;
	protected final BundleModuleRegistry resourceRegistry;	
	
	protected AbstractBundleResource(IPath path, BundleModuleRegistry bundleRegistry) {
		if(path == null || path.isEmpty() || !path.isValidPath(path.toString())) {
			throw new IllegalArgumentException("Invalid bundle resource path: " + path); //$NON-NLS-1$
		}
		
		this.resourcePath = path;
		this.resourceRegistry = bundleRegistry;
	}
	
	public String getBundleSymbolicName() {
		return resourceRegistry.getBundleSymbolicName();
	}
	
	protected IPath getPath() {
		return resourcePath;
	}	

	public String getFullPath() {
		return resourcePath.toOSString(); 
	}

	public String getName() {
		String name = resourcePath.lastSegment();
		return name != null ? name : ""; //$NON-NLS-1$
	}

	@Override
	public String toString() {	
		return "platform:/plugin/" + resourceRegistry.getBundleSymbolicName() + "/" + resourcePath.toString(); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	@Override
	public boolean equals(Object another) {
		if(another instanceof AbstractBundleResource) {
			AbstractBundleResource res = (AbstractBundleResource)another;
			return resourcePath.equals(res.resourcePath) && 
					resourceRegistry.getBundleSymbolicName().equals(
							res.resourceRegistry.getBundleSymbolicName()); 
		}
		
		return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + resourcePath.hashCode(); 
		result = 37 * result + resourceRegistry.getBundleSymbolicName().hashCode();
		return  result;
	}	
}